<?php

namespace Beo\Whitepaper\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Beo\Whitepaper\Forms\Whitepaper as WhitepaperForm;
use Beo\Whitepaper\Http\Requests\WhitepaperRequest;
use Beo\Whitepaper\Http\Resources\WhitepaperResource;
use Beo\Whitepaper\Http\Resources\WhitepapersCollection;
use Beo\Whitepaper\Models\Whitepaper;
use Beo\Whitepaper\Scopes\WhitepaperResourceScope;


/**
 * Resource controller class for whitepaper.
 */
class WhitepaperResourceController extends BaseController
{

    /**
     * Initialize whitepaper resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = WhitepaperForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('beo.whitepaper.modules'), 'whitepaper', guard_url('whitepaper'));
    }

    /**
     * Display a list of whitepaper.
     *
     * @return Response
     */
    public function index(WhitepaperRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Whitepaper::pushScope(new RequestScope())
            ->pushScope(new WhitepaperResourceScope())
            ->simplePaginate($pageLimit);

        $data = new WhitepapersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('whitepaper::whitepaper.names'))
            ->view('whitepaper::whitepaper.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display whitepaper.
     *
     * @param Request $request
     * @param Model   $whitepaper
     *
     * @return Response
     */
    public function show(WhitepaperRequest $request, Whitepaper $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WhitepaperResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('whitepaper::whitepaper.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('whitepaper::whitepaper.show')
            ->output();

    }

    /**
     * Show the form for creating a new whitepaper.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(WhitepaperRequest $request, Whitepaper $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WhitepaperResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('whitepaper::whitepaper.name'))
            ->view('whitepaper::whitepaper.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new whitepaper.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(WhitepaperRequest $request, Whitepaper $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new WhitepaperResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('whitepaper::whitepaper.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('whitepaper/whitepaper/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/whitepaper/whitepaper'))
                ->redirect();
        }

    }

    /**
     * Show whitepaper for editing.
     *
     * @param Request $request
     * @param Model   $whitepaper
     *
     * @return Response
     */
    public function edit(WhitepaperRequest $request, Whitepaper $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WhitepaperResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('whitepaper::whitepaper.name'))
            ->view('whitepaper::whitepaper.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the whitepaper.
     *
     * @param Request $request
     * @param Model   $whitepaper
     *
     * @return Response
     */
    public function update(WhitepaperRequest $request, Whitepaper $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new WhitepaperResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('whitepaper::whitepaper.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('whitepaper/whitepaper/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('whitepaper/whitepaper/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the whitepaper.
     *
     * @param Model   $whitepaper
     *
     * @return Response
     */
    public function destroy(WhitepaperRequest $request, Whitepaper $model)
    {
        try {
            $model->delete();
            $data = new WhitepaperResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('whitepaper::whitepaper.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('whitepaper/whitepaper/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('whitepaper/whitepaper/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
